package eu.mhsl.minenet.minigames.instance.game.stateless.types.tetris.game;

import eu.mhsl.minenet.minigames.instance.game.stateless.StatelessGame;
import eu.mhsl.minenet.minigames.util.BatchUtil;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.instance.batch.AbsoluteBlockBatch;
import net.minestom.server.instance.block.Block;

public class Tetromino {
    private final Shape shape;

    private int[][] shapeArray;

    public enum Shape {
        I,
        J,
        L,
        O,
        S,
        T,
        Z
    }

    public Tetromino(Shape shape) {
        this.shape = shape;

        switch (this.shape) {
            case I -> shapeArray = new int[][]{{0, 0, 0, 0}, {1, 1, 1, 1}, {0, 0, 0, 0}, {0, 0, 0, 0}};
            case J -> shapeArray = new int[][]{{1,0,0}, {1,1,1}, {0,0,0}};
            case L -> shapeArray = new int[][]{{0,0,1}, {1,1,1}, {0,0,0}};
            case O -> shapeArray = new int[][]{{1,1}, {1,1}};
            case S -> shapeArray = new int[][]{{0,1,1}, {1,1,0}, {0,0,0}};
            case T -> shapeArray = new int[][]{{0,1,0}, {1,1,1}, {0,0,0}};
            case Z -> shapeArray = new int[][]{{1,1,0}, {0,1,1}, {0,0,0}};
        }
    }

    private int[][] getTurnedShapeArray(boolean clockwise) {
        int iterations = 1;
        if(!clockwise) iterations = 3;

        int arrayLength = this.shapeArray.length;
        int[][] returnArray = new int[arrayLength][arrayLength];

        for(int k=0; k<iterations; k++) {
            for(int i=0; i<arrayLength; i++) {
                for(int j=0; j<arrayLength; j++) {
                    returnArray[i][arrayLength-1-j] = this.shapeArray[j][i];
                }
            }
        }

        return returnArray;
    }

    public void draw(StatelessGame instance, Pos spawnPosition) {
        int arrayLength = this.shapeArray.length;

        AbsoluteBlockBatch batch = new AbsoluteBlockBatch();

        for(int x=-1; x<arrayLength-1; x++) {
            for(int y=-1; y<arrayLength-1; y++) {
                batch.setBlock(spawnPosition.add(x, y, 0), Block.WHITE_CONCRETE);
            }
        }

        BatchUtil.loadAndApplyBatch(batch, instance, () -> {});
    }
}
